require "global_shared_db/version"

require 'active_record'
require 'active_support'
require 'active_support/core_ext'

require 'globalize'

module GlobalSharedDb

  class << self
    attr_reader :db_config

    def root
      @root ||= File.expand_path('../../', __FILE__)
    end

    def env
      @env ||= ENV['RACK_ENV'] || ENV['RAILS_ENV'] || 'development'
    end

    def setup(config)
      case config
      when Hash
        @db_config = config.stringify_keys
      else
        raise "Invalid config"
      end
      establish_connection(config)
    end

    def establish_connection(spec)
      if defined?(Octopus)
        GlobalSharedDb::BaseDb.octopus_establish_connection(spec)
      elsif defined?(DbCharmer)
        GlobalSharedDb::BaseDb.db_magic :connection => spec
      else
        GlobalSharedDb::BaseDb.establish_connection(spec)
      end
    end

    private

    def valid_config?(config)
      config[:adapter] && config[:host] && config[:database]
    rescue
      false
    end
  end

  Globalize.translations_field = :translations

  (
    Dir[File.expand_path('../global_shared_db/*.rb', __FILE__)] +
    Dir[File.expand_path('../global_shared_db/models/*.rb', __FILE__)]
  ).each do |filepath|
    filename = File.basename(filepath).gsub(/\.rb$/, '')
    autoload filename.camelize, filepath
  end

  module Concerns
    Dir[File.expand_path('../global_shared_db/concerns/*.rb', __FILE__)].each do |filepath|
      filename = File.basename(filepath).gsub(/\.rb$/, '')
      autoload filename.camelize, filepath
    end
  end
end
