module GlobalSharedDb
  module Concerns
    module AttachmentField
      extend ActiveSupport::Concern

      class InvalidCdnError < StandardError; end

      module ClassMethods
        def attachment_fields(*ks)
          ks ||= []

          mod = Module.new do
            ks.each do |k|
              define_method("#{k}_url") do
                v = public_send(k)
                return v if v.blank?

                attachment_to_url(v)
              end
            end
          end

          include mod
        end
      end # ClassMethods

      included do
        private
        def attachment_to_url(resource)
          cdn = GlobalSharedDb::Settings.base['cdn']
          raise InvalidCdnError if cdn.blank?

          "#{cdn}/#{resource}"
        end
      end

    end
  end
end